SELECT pg_catalog.pg_extension_config_dump('us_lex', 'WHERE is_custom');
SELECT pg_catalog.pg_extension_config_dump('us_rules', 'WHERE is_custom');
SELECT pg_catalog.pg_extension_config_dump('us_gaz', 'WHERE is_custom');
