/*
 * Country aliases for parse_address(), generated from iso-codes and
 * a small compatibility alias set. Keep this list sorted by alias.
 */
{"ABW", "AW"},
{"AD", "AD"},
{"AE", "AE"},
{"AF", "AF"},
{"AFG", "AF"},
{"AFGHANISTAN", "AF"},
{"AG", "AG"},
{"AGO", "AO"},
{"AI", "AI"},
{"AIA", "AI"},
{"AL", "AL"},
{"ALA", "AX"},
{"ALAND ISLANDS", "AX"},
{"ALB", "AL"},
{"ALBANIA", "AL"},
{"ALGERIA", "DZ"},
{"AM", "AM"},
{"AMERICAN SAMOA", "AS"},
{"AND", "AD"},
{"ANDORRA", "AD"},
{"ANGOLA", "AO"},
{"ANGUILLA", "AI"},
{"ANTARCTICA", "AQ"},
{"ANTIGUA AND BARBUDA", "AG"},
{"AO", "AO"},
{"AQ", "AQ"},
{"AR", "AR"},
{"ARAB REPUBLIC OF EGYPT", "EG"},
{"ARE", "AE"},
{"ARG", "AR"},
{"ARGENTINA", "AR"},
{"ARGENTINE REPUBLIC", "AR"},
{"ARM", "AM"},
{"ARMENIA", "AM"},
{"ARUBA", "AW"},
{"AS", "AS"},
{"ASM", "AS"},
{"AT", "AT"},
{"ATA", "AQ"},
{"ATF", "TF"},
{"ATG", "AG"},
{"AU", "AU"},
{"AUS", "AU"},
{"AUSTRALIA", "AU"},
{"AUSTRIA", "AT"},
{"AUT", "AT"},
{"AW", "AW"},
{"AX", "AX"},
{"AZ", "AZ"},
{"AZE", "AZ"},
{"AZERBAIJAN", "AZ"},
{"BA", "BA"},
{"BAHAMAS", "BS"},
{"BAHRAIN", "BH"},
{"BANGLADESH", "BD"},
{"BARBADOS", "BB"},
{"BB", "BB"},
{"BD", "BD"},
{"BDI", "BI"},
{"BE", "BE"},
{"BEL", "BE"},
{"BELARUS", "BY"},
{"BELGIUM", "BE"},
{"BELIZE", "BZ"},
{"BEN", "BJ"},
{"BENIN", "BJ"},
{"BERMUDA", "BM"},
{"BES", "BQ"},
{"BF", "BF"},
{"BFA", "BF"},
{"BG", "BG"},
{"BGD", "BD"},
{"BGR", "BG"},
{"BH", "BH"},
{"BHR", "BH"},
{"BHS", "BS"},
{"BHUTAN", "BT"},
{"BI", "BI"},
{"BIH", "BA"},
{"BJ", "BJ"},
{"BL", "BL"},
{"BLM", "BL"},
{"BLR", "BY"},
{"BLZ", "BZ"},
{"BM", "BM"},
{"BMU", "BM"},
{"BN", "BN"},
{"BO", "BO"},
{"BOL", "BO"},
{"BOLIVARIAN REPUBLIC OF VENEZUELA", "VE"},
{"BOLIVIA", "BO"},
{"BOLIVIA PLURINATIONAL STATE OF", "BO"},
{"BONAIRE SINT EUSTATIUS AND SABA", "BQ"},
{"BOSNIA AND HERZEGOVINA", "BA"},
{"BOSNIA HERZEGOVINA", "BA"},
{"BOTSWANA", "BW"},
{"BOUVET ISLAND", "BV"},
{"BQ", "BQ"},
{"BR", "BR"},
{"BRA", "BR"},
{"BRAZIL", "BR"},
{"BRB", "BB"},
{"BRITAIN", "GB"},
{"BRITISH INDIAN OCEAN TERRITORY", "IO"},
{"BRITISH VIRGIN ISLANDS", "VG"},
{"BRN", "BN"},
{"BRUNEI DARUSSALAM", "BN"},
{"BS", "BS"},
{"BT", "BT"},
{"BTN", "BT"},
{"BULGARIA", "BG"},
{"BURKINA FASO", "BF"},
{"BURMA", "MM"},
{"BURUNDI", "BI"},
{"BV", "BV"},
{"BVT", "BV"},
{"BW", "BW"},
{"BWA", "BW"},
{"BY", "BY"},
{"BZ", "BZ"},
{"CA", "CA"},
{"CABO VERDE", "CV"},
{"CAF", "CF"},
{"CAMBODIA", "KH"},
{"CAMEROON", "CM"},
{"CAN", "CA"},
{"CANADA", "CA"},
{"CAYMAN ISLANDS", "KY"},
{"CC", "CC"},
{"CCK", "CC"},
{"CD", "CD"},
{"CENTRAL AFRICAN REPUBLIC", "CF"},
{"CF", "CF"},
{"CG", "CG"},
{"CH", "CH"},
{"CHAD", "TD"},
{"CHE", "CH"},
{"CHILE", "CL"},
{"CHINA", "CN"},
{"CHL", "CL"},
{"CHN", "CN"},
{"CHRISTMAS ISLAND", "CX"},
{"CI", "CI"},
{"CIV", "CI"},
{"CK", "CK"},
{"CL", "CL"},
{"CM", "CM"},
{"CMR", "CM"},
{"CN", "CN"},
{"CO", "CO"},
{"COCOS KEELING ISLANDS", "CC"},
{"COD", "CD"},
{"COG", "CG"},
{"COK", "CK"},
{"COL", "CO"},
{"COLOMBIA", "CO"},
{"COM", "KM"},
{"COMMONWEALTH OF DOMINICA", "DM"},
{"COMMONWEALTH OF THE BAHAMAS", "BS"},
{"COMMONWEALTH OF THE NORTHERN MARIANA ISLANDS", "MP"},
{"COMOROS", "KM"},
{"CONGO", "CG"},
{"CONGO THE DEMOCRATIC REPUBLIC OF THE", "CD"},
{"COOK ISLANDS", "CK"},
{"COSTA RICA", "CR"},
{"COTE D IVOIRE", "CI"},
{"CPV", "CV"},
{"CR", "CR"},
{"CRI", "CR"},
{"CROATIA", "HR"},
{"CU", "CU"},
{"CUB", "CU"},
{"CUBA", "CU"},
{"CURACAO", "CW"},
{"CUW", "CW"},
{"CV", "CV"},
{"CW", "CW"},
{"CX", "CX"},
{"CXR", "CX"},
{"CY", "CY"},
{"CYM", "KY"},
{"CYP", "CY"},
{"CYPRUS", "CY"},
{"CZ", "CZ"},
{"CZE", "CZ"},
{"CZECH REPUBLIC", "CZ"},
{"CZECHIA", "CZ"},
{"DE", "DE"},
{"DEMOCRATIC PEOPLE S REPUBLIC OF KOREA", "KP"},
{"DEMOCRATIC REPUBLIC OF SAO TOME AND PRINCIPE", "ST"},
{"DEMOCRATIC REPUBLIC OF TIMOR LESTE", "TL"},
{"DEMOCRATIC SOCIALIST REPUBLIC OF SRI LANKA", "LK"},
{"DENMARK", "DK"},
{"DEU", "DE"},
{"DJ", "DJ"},
{"DJI", "DJ"},
{"DJIBOUTI", "DJ"},
{"DK", "DK"},
{"DM", "DM"},
{"DMA", "DM"},
{"DNK", "DK"},
{"DO", "DO"},
{"DOM", "DO"},
{"DOMINICA", "DM"},
{"DOMINICAN REPUBLIC", "DO"},
{"DZ", "DZ"},
{"DZA", "DZ"},
{"EAST TIMOR", "TL"},
{"EASTERN REPUBLIC OF URUGUAY", "UY"},
{"EC", "EC"},
{"ECU", "EC"},
{"ECUADOR", "EC"},
{"EE", "EE"},
{"EG", "EG"},
{"EGY", "EG"},
{"EGYPT", "EG"},
{"EH", "EH"},
{"EL SALVADOR", "SV"},
{"EQUATORIAL GUINEA", "GQ"},
{"ER", "ER"},
{"ERI", "ER"},
{"ERITREA", "ER"},
{"ES", "ES"},
{"ESH", "EH"},
{"ESP", "ES"},
{"EST", "EE"},
{"ESTONIA", "EE"},
{"ESWATINI", "SZ"},
{"ET", "ET"},
{"ETH", "ET"},
{"ETHIOPIA", "ET"},
{"FALKLAND ISLANDS MALVINAS", "FK"},
{"FAROE ISLANDS", "FO"},
{"FEDERAL DEMOCRATIC REPUBLIC OF ETHIOPIA", "ET"},
{"FEDERAL DEMOCRATIC REPUBLIC OF NEPAL", "NP"},
{"FEDERAL REPUBLIC OF GERMANY", "DE"},
{"FEDERAL REPUBLIC OF NIGERIA", "NG"},
{"FEDERAL REPUBLIC OF SOMALIA", "SO"},
{"FEDERATED STATES OF MICRONESIA", "FM"},
{"FEDERATIVE REPUBLIC OF BRAZIL", "BR"},
{"FI", "FI"},
{"FIJI", "FJ"},
{"FIN", "FI"},
{"FINLAND", "FI"},
{"FJ", "FJ"},
{"FJI", "FJ"},
{"FK", "FK"},
{"FLK", "FK"},
{"FM", "FM"},
{"FO", "FO"},
{"FR", "FR"},
{"FRA", "FR"},
{"FRANCE", "FR"},
{"FRENCH GUIANA", "GF"},
{"FRENCH POLYNESIA", "PF"},
{"FRENCH REPUBLIC", "FR"},
{"FRENCH SOUTHERN TERRITORIES", "TF"},
{"FRO", "FO"},
{"FSM", "FM"},
{"GA", "GA"},
{"GAB", "GA"},
{"GABON", "GA"},
{"GABONESE REPUBLIC", "GA"},
{"GAMBIA", "GM"},
{"GB", "GB"},
{"GBR", "GB"},
{"GD", "GD"},
{"GE", "GE"},
{"GEO", "GE"},
{"GEORGIA", "GE"},
{"GERMANY", "DE"},
{"GF", "GF"},
{"GG", "GG"},
{"GGY", "GG"},
{"GH", "GH"},
{"GHA", "GH"},
{"GHANA", "GH"},
{"GI", "GI"},
{"GIB", "GI"},
{"GIBRALTAR", "GI"},
{"GIN", "GN"},
{"GL", "GL"},
{"GLP", "GP"},
{"GM", "GM"},
{"GMB", "GM"},
{"GN", "GN"},
{"GNB", "GW"},
{"GNQ", "GQ"},
{"GP", "GP"},
{"GQ", "GQ"},
{"GR", "GR"},
{"GRAND DUCHY OF LUXEMBOURG", "LU"},
{"GRC", "GR"},
{"GRD", "GD"},
{"GREAT BRITAIN", "GB"},
{"GREECE", "GR"},
{"GREENLAND", "GL"},
{"GRENADA", "GD"},
{"GRL", "GL"},
{"GS", "GS"},
{"GT", "GT"},
{"GTM", "GT"},
{"GU", "GU"},
{"GUADELOUPE", "GP"},
{"GUAM", "GU"},
{"GUATEMALA", "GT"},
{"GUERNSEY", "GG"},
{"GUF", "GF"},
{"GUINEA", "GN"},
{"GUINEA BISSAU", "GW"},
{"GUM", "GU"},
{"GUY", "GY"},
{"GUYANA", "GY"},
{"GW", "GW"},
{"GY", "GY"},
{"HAITI", "HT"},
{"HASHEMITE KINGDOM OF JORDAN", "JO"},
{"HEARD ISLAND AND MCDONALD ISLANDS", "HM"},
{"HELLENIC REPUBLIC", "GR"},
{"HK", "HK"},
{"HKG", "HK"},
{"HM", "HM"},
{"HMD", "HM"},
{"HN", "HN"},
{"HND", "HN"},
{"HOLY SEE VATICAN CITY STATE", "VA"},
{"HONDURAS", "HN"},
{"HONG KONG", "HK"},
{"HONG KONG SPECIAL ADMINISTRATIVE REGION OF CHINA", "HK"},
{"HR", "HR"},
{"HRV", "HR"},
{"HT", "HT"},
{"HTI", "HT"},
{"HU", "HU"},
{"HUN", "HU"},
{"HUNGARY", "HU"},
{"ICELAND", "IS"},
{"ID", "ID"},
{"IDN", "ID"},
{"IE", "IE"},
{"IL", "IL"},
{"IM", "IM"},
{"IMN", "IM"},
{"IN", "IN"},
{"IND", "IN"},
{"INDEPENDENT STATE OF PAPUA NEW GUINEA", "PG"},
{"INDEPENDENT STATE OF SAMOA", "WS"},
{"INDIA", "IN"},
{"INDONESIA", "ID"},
{"IO", "IO"},
{"IOT", "IO"},
{"IQ", "IQ"},
{"IR", "IR"},
{"IRAN", "IR"},
{"IRAN ISLAMIC REPUBLIC OF", "IR"},
{"IRAQ", "IQ"},
{"IRELAND", "IE"},
{"IRL", "IE"},
{"IRN", "IR"},
{"IRQ", "IQ"},
{"IS", "IS"},
{"ISL", "IS"},
{"ISLAMIC REPUBLIC OF AFGHANISTAN", "AF"},
{"ISLAMIC REPUBLIC OF IRAN", "IR"},
{"ISLAMIC REPUBLIC OF MAURITANIA", "MR"},
{"ISLAMIC REPUBLIC OF PAKISTAN", "PK"},
{"ISLE OF MAN", "IM"},
{"ISR", "IL"},
{"ISRAEL", "IL"},
{"IT", "IT"},
{"ITA", "IT"},
{"ITALIAN REPUBLIC", "IT"},
{"ITALY", "IT"},
{"IVORY COAST", "CI"},
{"JAM", "JM"},
{"JAMAICA", "JM"},
{"JAPAN", "JP"},
{"JE", "JE"},
{"JERSEY", "JE"},
{"JEY", "JE"},
{"JM", "JM"},
{"JO", "JO"},
{"JOR", "JO"},
{"JORDAN", "JO"},
{"JP", "JP"},
{"JPN", "JP"},
{"KAZ", "KZ"},
{"KAZAKHSTAN", "KZ"},
{"KE", "KE"},
{"KEN", "KE"},
{"KENYA", "KE"},
{"KG", "KG"},
{"KGZ", "KG"},
{"KH", "KH"},
{"KHM", "KH"},
{"KI", "KI"},
{"KINGDOM OF BAHRAIN", "BH"},
{"KINGDOM OF BELGIUM", "BE"},
{"KINGDOM OF BHUTAN", "BT"},
{"KINGDOM OF CAMBODIA", "KH"},
{"KINGDOM OF DENMARK", "DK"},
{"KINGDOM OF ESWATINI", "SZ"},
{"KINGDOM OF LESOTHO", "LS"},
{"KINGDOM OF MOROCCO", "MA"},
{"KINGDOM OF NORWAY", "NO"},
{"KINGDOM OF SAUDI ARABIA", "SA"},
{"KINGDOM OF SPAIN", "ES"},
{"KINGDOM OF SWEDEN", "SE"},
{"KINGDOM OF THAILAND", "TH"},
{"KINGDOM OF THE NETHERLANDS", "NL"},
{"KINGDOM OF TONGA", "TO"},
{"KIR", "KI"},
{"KIRIBATI", "KI"},
{"KM", "KM"},
{"KN", "KN"},
{"KNA", "KN"},
{"KOR", "KR"},
{"KOREA DEMOCRATIC PEOPLE S REPUBLIC OF", "KP"},
{"KOREA REPUBLIC OF", "KR"},
{"KP", "KP"},
{"KR", "KR"},
{"KUWAIT", "KW"},
{"KW", "KW"},
{"KWT", "KW"},
{"KY", "KY"},
{"KYRGYZ REPUBLIC", "KG"},
{"KYRGYZSTAN", "KG"},
{"KZ", "KZ"},
{"LA", "LA"},
{"LAO", "LA"},
{"LAO PEOPLE S DEMOCRATIC REPUBLIC", "LA"},
{"LAOS", "LA"},
{"LATVIA", "LV"},
{"LB", "LB"},
{"LBN", "LB"},
{"LBR", "LR"},
{"LBY", "LY"},
{"LC", "LC"},
{"LCA", "LC"},
{"LEBANESE REPUBLIC", "LB"},
{"LEBANON", "LB"},
{"LESOTHO", "LS"},
{"LI", "LI"},
{"LIBERIA", "LR"},
{"LIBYA", "LY"},
{"LIE", "LI"},
{"LIECHTENSTEIN", "LI"},
{"LITHUANIA", "LT"},
{"LK", "LK"},
{"LKA", "LK"},
{"LR", "LR"},
{"LS", "LS"},
{"LSO", "LS"},
{"LT", "LT"},
{"LTU", "LT"},
{"LU", "LU"},
{"LUX", "LU"},
{"LUXEMBOURG", "LU"},
{"LV", "LV"},
{"LVA", "LV"},
{"LY", "LY"},
{"MA", "MA"},
{"MAC", "MO"},
{"MACAO", "MO"},
{"MACAO SPECIAL ADMINISTRATIVE REGION OF CHINA", "MO"},
{"MADAGASCAR", "MG"},
{"MAF", "MF"},
{"MALAWI", "MW"},
{"MALAYSIA", "MY"},
{"MALDIVES", "MV"},
{"MALI", "ML"},
{"MALTA", "MT"},
{"MAR", "MA"},
{"MARSHALL ISLANDS", "MH"},
{"MARTINIQUE", "MQ"},
{"MAURITANIA", "MR"},
{"MAURITIUS", "MU"},
{"MAYOTTE", "YT"},
{"MC", "MC"},
{"MCO", "MC"},
{"MD", "MD"},
{"MDA", "MD"},
{"MDG", "MG"},
{"MDV", "MV"},
{"ME", "ME"},
{"MEX", "MX"},
{"MEXICO", "MX"},
{"MF", "MF"},
{"MG", "MG"},
{"MH", "MH"},
{"MHL", "MH"},
{"MICRONESIA", "FM"},
{"MICRONESIA FEDERATED STATES OF", "FM"},
{"MK", "MK"},
{"MKD", "MK"},
{"ML", "ML"},
{"MLI", "ML"},
{"MLT", "MT"},
{"MM", "MM"},
{"MMR", "MM"},
{"MN", "MN"},
{"MNE", "ME"},
{"MNG", "MN"},
{"MNP", "MP"},
{"MO", "MO"},
{"MOLDOVA", "MD"},
{"MOLDOVA REPUBLIC OF", "MD"},
{"MONACO", "MC"},
{"MONGOLIA", "MN"},
{"MONTENEGRO", "ME"},
{"MONTSERRAT", "MS"},
{"MOROCCO", "MA"},
{"MOZ", "MZ"},
{"MOZAMBIQUE", "MZ"},
{"MP", "MP"},
{"MQ", "MQ"},
{"MR", "MR"},
{"MRT", "MR"},
{"MS", "MS"},
{"MSR", "MS"},
{"MT", "MT"},
{"MTQ", "MQ"},
{"MU", "MU"},
{"MUS", "MU"},
{"MV", "MV"},
{"MW", "MW"},
{"MWI", "MW"},
{"MX", "MX"},
{"MY", "MY"},
{"MYANMAR", "MM"},
{"MYS", "MY"},
{"MYT", "YT"},
{"MZ", "MZ"},
{"NA", "NA"},
{"NAM", "NA"},
{"NAMIBIA", "NA"},
{"NAURU", "NR"},
{"NC", "NC"},
{"NCL", "NC"},
{"NE", "NE"},
{"NEPAL", "NP"},
{"NER", "NE"},
{"NETHERLANDS", "NL"},
{"NEW CALEDONIA", "NC"},
{"NEW ZEALAND", "NZ"},
{"NF", "NF"},
{"NFK", "NF"},
{"NG", "NG"},
{"NGA", "NG"},
{"NI", "NI"},
{"NIC", "NI"},
{"NICARAGUA", "NI"},
{"NIGER", "NE"},
{"NIGERIA", "NG"},
{"NIU", "NU"},
{"NIUE", "NU"},
{"NL", "NL"},
{"NLD", "NL"},
{"NO", "NO"},
{"NOR", "NO"},
{"NORFOLK ISLAND", "NF"},
{"NORTH KOREA", "KP"},
{"NORTH MACEDONIA", "MK"},
{"NORTHERN MARIANA ISLANDS", "MP"},
{"NORWAY", "NO"},
{"NP", "NP"},
{"NPL", "NP"},
{"NR", "NR"},
{"NRU", "NR"},
{"NU", "NU"},
{"NZ", "NZ"},
{"NZL", "NZ"},
{"OM", "OM"},
{"OMAN", "OM"},
{"OMN", "OM"},
{"PA", "PA"},
{"PAK", "PK"},
{"PAKISTAN", "PK"},
{"PALAU", "PW"},
{"PALESTINE", "PS"},
{"PALESTINE STATE OF", "PS"},
{"PAN", "PA"},
{"PANAMA", "PA"},
{"PAPUA NEW GUINEA", "PG"},
{"PARAGUAY", "PY"},
{"PCN", "PN"},
{"PE", "PE"},
{"PEOPLE S DEMOCRATIC REPUBLIC OF ALGERIA", "DZ"},
{"PEOPLE S REPUBLIC OF BANGLADESH", "BD"},
{"PEOPLE S REPUBLIC OF CHINA", "CN"},
{"PER", "PE"},
{"PERU", "PE"},
{"PF", "PF"},
{"PG", "PG"},
{"PH", "PH"},
{"PHILIPPINES", "PH"},
{"PHL", "PH"},
{"PITCAIRN", "PN"},
{"PK", "PK"},
{"PL", "PL"},
{"PLURINATIONAL STATE OF BOLIVIA", "BO"},
{"PLW", "PW"},
{"PM", "PM"},
{"PN", "PN"},
{"PNG", "PG"},
{"POL", "PL"},
{"POLAND", "PL"},
{"PORTUGAL", "PT"},
{"PORTUGUESE REPUBLIC", "PT"},
{"PR", "PR"},
{"PRI", "PR"},
{"PRINCIPALITY OF ANDORRA", "AD"},
{"PRINCIPALITY OF LIECHTENSTEIN", "LI"},
{"PRINCIPALITY OF MONACO", "MC"},
{"PRK", "KP"},
{"PRT", "PT"},
{"PRY", "PY"},
{"PS", "PS"},
{"PSE", "PS"},
{"PT", "PT"},
{"PUERTO RICO", "PR"},
{"PW", "PW"},
{"PY", "PY"},
{"PYF", "PF"},
{"QA", "QA"},
{"QAT", "QA"},
{"QATAR", "QA"},
{"RE", "RE"},
{"REPUBLIC OF ALBANIA", "AL"},
{"REPUBLIC OF ANGOLA", "AO"},
{"REPUBLIC OF ARMENIA", "AM"},
{"REPUBLIC OF AUSTRIA", "AT"},
{"REPUBLIC OF AZERBAIJAN", "AZ"},
{"REPUBLIC OF BELARUS", "BY"},
{"REPUBLIC OF BENIN", "BJ"},
{"REPUBLIC OF BOSNIA AND HERZEGOVINA", "BA"},
{"REPUBLIC OF BOTSWANA", "BW"},
{"REPUBLIC OF BULGARIA", "BG"},
{"REPUBLIC OF BURUNDI", "BI"},
{"REPUBLIC OF CABO VERDE", "CV"},
{"REPUBLIC OF CAMEROON", "CM"},
{"REPUBLIC OF CHAD", "TD"},
{"REPUBLIC OF CHILE", "CL"},
{"REPUBLIC OF COLOMBIA", "CO"},
{"REPUBLIC OF COSTA RICA", "CR"},
{"REPUBLIC OF COTE D IVOIRE", "CI"},
{"REPUBLIC OF CROATIA", "HR"},
{"REPUBLIC OF CUBA", "CU"},
{"REPUBLIC OF CYPRUS", "CY"},
{"REPUBLIC OF DJIBOUTI", "DJ"},
{"REPUBLIC OF ECUADOR", "EC"},
{"REPUBLIC OF EL SALVADOR", "SV"},
{"REPUBLIC OF EQUATORIAL GUINEA", "GQ"},
{"REPUBLIC OF ESTONIA", "EE"},
{"REPUBLIC OF FIJI", "FJ"},
{"REPUBLIC OF FINLAND", "FI"},
{"REPUBLIC OF GHANA", "GH"},
{"REPUBLIC OF GUATEMALA", "GT"},
{"REPUBLIC OF GUINEA", "GN"},
{"REPUBLIC OF GUINEA BISSAU", "GW"},
{"REPUBLIC OF GUYANA", "GY"},
{"REPUBLIC OF HAITI", "HT"},
{"REPUBLIC OF HONDURAS", "HN"},
{"REPUBLIC OF ICELAND", "IS"},
{"REPUBLIC OF INDIA", "IN"},
{"REPUBLIC OF INDONESIA", "ID"},
{"REPUBLIC OF IRAQ", "IQ"},
{"REPUBLIC OF KAZAKHSTAN", "KZ"},
{"REPUBLIC OF KENYA", "KE"},
{"REPUBLIC OF KIRIBATI", "KI"},
{"REPUBLIC OF LATVIA", "LV"},
{"REPUBLIC OF LIBERIA", "LR"},
{"REPUBLIC OF LITHUANIA", "LT"},
{"REPUBLIC OF MADAGASCAR", "MG"},
{"REPUBLIC OF MALAWI", "MW"},
{"REPUBLIC OF MALDIVES", "MV"},
{"REPUBLIC OF MALI", "ML"},
{"REPUBLIC OF MALTA", "MT"},
{"REPUBLIC OF MAURITIUS", "MU"},
{"REPUBLIC OF MOLDOVA", "MD"},
{"REPUBLIC OF MOZAMBIQUE", "MZ"},
{"REPUBLIC OF MYANMAR", "MM"},
{"REPUBLIC OF NAMIBIA", "NA"},
{"REPUBLIC OF NAURU", "NR"},
{"REPUBLIC OF NICARAGUA", "NI"},
{"REPUBLIC OF NORTH MACEDONIA", "MK"},
{"REPUBLIC OF PALAU", "PW"},
{"REPUBLIC OF PANAMA", "PA"},
{"REPUBLIC OF PARAGUAY", "PY"},
{"REPUBLIC OF PERU", "PE"},
{"REPUBLIC OF POLAND", "PL"},
{"REPUBLIC OF SAN MARINO", "SM"},
{"REPUBLIC OF SENEGAL", "SN"},
{"REPUBLIC OF SERBIA", "RS"},
{"REPUBLIC OF SEYCHELLES", "SC"},
{"REPUBLIC OF SIERRA LEONE", "SL"},
{"REPUBLIC OF SINGAPORE", "SG"},
{"REPUBLIC OF SLOVENIA", "SI"},
{"REPUBLIC OF SOUTH AFRICA", "ZA"},
{"REPUBLIC OF SOUTH SUDAN", "SS"},
{"REPUBLIC OF SURINAME", "SR"},
{"REPUBLIC OF TAJIKISTAN", "TJ"},
{"REPUBLIC OF THE CONGO", "CG"},
{"REPUBLIC OF THE GAMBIA", "GM"},
{"REPUBLIC OF THE MARSHALL ISLANDS", "MH"},
{"REPUBLIC OF THE NIGER", "NE"},
{"REPUBLIC OF THE PHILIPPINES", "PH"},
{"REPUBLIC OF THE SUDAN", "SD"},
{"REPUBLIC OF TRINIDAD AND TOBAGO", "TT"},
{"REPUBLIC OF TUNISIA", "TN"},
{"REPUBLIC OF TURKIYE", "TR"},
{"REPUBLIC OF UGANDA", "UG"},
{"REPUBLIC OF UZBEKISTAN", "UZ"},
{"REPUBLIC OF VANUATU", "VU"},
{"REPUBLIC OF YEMEN", "YE"},
{"REPUBLIC OF ZAMBIA", "ZM"},
{"REPUBLIC OF ZIMBABWE", "ZW"},
{"REU", "RE"},
{"REUNION", "RE"},
{"RO", "RO"},
{"ROMANIA", "RO"},
{"ROU", "RO"},
{"RS", "RS"},
{"RU", "RU"},
{"RUS", "RU"},
{"RUSSIA", "RU"},
{"RUSSIAN FEDERATION", "RU"},
{"RW", "RW"},
{"RWA", "RW"},
{"RWANDA", "RW"},
{"RWANDESE REPUBLIC", "RW"},
{"SA", "SA"},
{"SAINT BARTHELEMY", "BL"},
{"SAINT HELENA ASCENSION AND TRISTAN DA CUNHA", "SH"},
{"SAINT KITTS AND NEVIS", "KN"},
{"SAINT LUCIA", "LC"},
{"SAINT MARTIN FRENCH PART", "MF"},
{"SAINT PIERRE AND MIQUELON", "PM"},
{"SAINT VINCENT AND THE GRENADINES", "VC"},
{"SAMOA", "WS"},
{"SAN MARINO", "SM"},
{"SAO TOME AND PRINCIPE", "ST"},
{"SAU", "SA"},
{"SAUDI ARABIA", "SA"},
{"SB", "SB"},
{"SC", "SC"},
{"SD", "SD"},
{"SDN", "SD"},
{"SE", "SE"},
{"SEN", "SN"},
{"SENEGAL", "SN"},
{"SERBIA", "RS"},
{"SEYCHELLES", "SC"},
{"SG", "SG"},
{"SGP", "SG"},
{"SGS", "GS"},
{"SH", "SH"},
{"SHN", "SH"},
{"SI", "SI"},
{"SIERRA LEONE", "SL"},
{"SINGAPORE", "SG"},
{"SINT MAARTEN DUTCH PART", "SX"},
{"SJ", "SJ"},
{"SJM", "SJ"},
{"SK", "SK"},
{"SL", "SL"},
{"SLB", "SB"},
{"SLE", "SL"},
{"SLOVAK REPUBLIC", "SK"},
{"SLOVAKIA", "SK"},
{"SLOVENIA", "SI"},
{"SLV", "SV"},
{"SM", "SM"},
{"SMR", "SM"},
{"SN", "SN"},
{"SO", "SO"},
{"SOCIALIST REPUBLIC OF VIET NAM", "VN"},
{"SOLOMON ISLANDS", "SB"},
{"SOM", "SO"},
{"SOMALIA", "SO"},
{"SOUTH AFRICA", "ZA"},
{"SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS", "GS"},
{"SOUTH KOREA", "KR"},
{"SOUTH SUDAN", "SS"},
{"SPAIN", "ES"},
{"SPM", "PM"},
{"SR", "SR"},
{"SRB", "RS"},
{"SRI LANKA", "LK"},
{"SS", "SS"},
{"SSD", "SS"},
{"ST", "ST"},
{"STATE OF ISRAEL", "IL"},
{"STATE OF KUWAIT", "KW"},
{"STATE OF QATAR", "QA"},
{"STP", "ST"},
{"SUDAN", "SD"},
{"SULTANATE OF OMAN", "OM"},
{"SUR", "SR"},
{"SURINAME", "SR"},
{"SV", "SV"},
{"SVALBARD AND JAN MAYEN", "SJ"},
{"SVK", "SK"},
{"SVN", "SI"},
{"SWE", "SE"},
{"SWEDEN", "SE"},
{"SWISS CONFEDERATION", "CH"},
{"SWITZERLAND", "CH"},
{"SWZ", "SZ"},
{"SX", "SX"},
{"SXM", "SX"},
{"SY", "SY"},
{"SYC", "SC"},
{"SYR", "SY"},
{"SYRIA", "SY"},
{"SYRIAN ARAB REPUBLIC", "SY"},
{"SZ", "SZ"},
{"TAIWAN", "TW"},
{"TAIWAN PROVINCE OF CHINA", "TW"},
{"TAJIKISTAN", "TJ"},
{"TANZANIA", "TZ"},
{"TANZANIA UNITED REPUBLIC OF", "TZ"},
{"TC", "TC"},
{"TCA", "TC"},
{"TCD", "TD"},
{"TD", "TD"},
{"TF", "TF"},
{"TG", "TG"},
{"TGO", "TG"},
{"TH", "TH"},
{"THA", "TH"},
{"THAILAND", "TH"},
{"THE STATE OF ERITREA", "ER"},
{"THE STATE OF PALESTINE", "PS"},
{"TIMOR LESTE", "TL"},
{"TJ", "TJ"},
{"TJK", "TJ"},
{"TK", "TK"},
{"TKL", "TK"},
{"TKM", "TM"},
{"TL", "TL"},
{"TLS", "TL"},
{"TM", "TM"},
{"TN", "TN"},
{"TO", "TO"},
{"TOGO", "TG"},
{"TOGOLESE REPUBLIC", "TG"},
{"TOKELAU", "TK"},
{"TON", "TO"},
{"TONGA", "TO"},
{"TR", "TR"},
{"TRINIDAD AND TOBAGO", "TT"},
{"TT", "TT"},
{"TTO", "TT"},
{"TUN", "TN"},
{"TUNISIA", "TN"},
{"TUR", "TR"},
{"TURKEY", "TR"},
{"TURKIYE", "TR"},
{"TURKMENISTAN", "TM"},
{"TURKS AND CAICOS ISLANDS", "TC"},
{"TUV", "TV"},
{"TUVALU", "TV"},
{"TV", "TV"},
{"TW", "TW"},
{"TWN", "TW"},
{"TZ", "TZ"},
{"TZA", "TZ"},
{"U K", "GB"},
{"U S", "US"},
{"U S A", "US"},
{"UA", "UA"},
{"UG", "UG"},
{"UGA", "UG"},
{"UGANDA", "UG"},
{"UK", "GB"},
{"UKR", "UA"},
{"UKRAINE", "UA"},
{"UM", "UM"},
{"UMI", "UM"},
{"UNION OF THE COMOROS", "KM"},
{"UNITED ARAB EMIRATES", "AE"},
{"UNITED KINGDOM", "GB"},
{"UNITED KINGDOM OF GREAT BRITAIN AND NORTHERN IRELAND", "GB"},
{"UNITED MEXICAN STATES", "MX"},
{"UNITED REPUBLIC OF TANZANIA", "TZ"},
{"UNITED STATES", "US"},
{"UNITED STATES MINOR OUTLYING ISLANDS", "UM"},
{"UNITED STATES OF AMERICA", "US"},
{"URUGUAY", "UY"},
{"URY", "UY"},
{"US", "US"},
{"USA", "US"},
{"UY", "UY"},
{"UZ", "UZ"},
{"UZB", "UZ"},
{"UZBEKISTAN", "UZ"},
{"VA", "VA"},
{"VANUATU", "VU"},
{"VAT", "VA"},
{"VATICAN CITY", "VA"},
{"VC", "VC"},
{"VCT", "VC"},
{"VE", "VE"},
{"VEN", "VE"},
{"VENEZUELA", "VE"},
{"VENEZUELA BOLIVARIAN REPUBLIC OF", "VE"},
{"VG", "VG"},
{"VGB", "VG"},
{"VI", "VI"},
{"VIET NAM", "VN"},
{"VIETNAM", "VN"},
{"VIR", "VI"},
{"VIRGIN ISLANDS BRITISH", "VG"},
{"VIRGIN ISLANDS OF THE UNITED STATES", "VI"},
{"VIRGIN ISLANDS U S", "VI"},
{"VN", "VN"},
{"VNM", "VN"},
{"VU", "VU"},
{"VUT", "VU"},
{"WALLIS AND FUTUNA", "WF"},
{"WESTERN SAHARA", "EH"},
{"WF", "WF"},
{"WLF", "WF"},
{"WS", "WS"},
{"WSM", "WS"},
{"YE", "YE"},
{"YEM", "YE"},
{"YEMEN", "YE"},
{"YT", "YT"},
{"ZA", "ZA"},
{"ZAF", "ZA"},
{"ZAMBIA", "ZM"},
{"ZIMBABWE", "ZW"},
{"ZM", "ZM"},
{"ZMB", "ZM"},
{"ZW", "ZW"},
{"ZWE", "ZW"},
